#!/bin/bash

# Author: Brian Harring <ferringb@gentoo.org>
# Script runs mirror-dist to cleanse/update distfiles

DEBUGGING=
#uncomment to enable debugging runs.
#DEBUGGING='yes'

export PYTHONWARNINGS=d
#PORTDIR=/home/gmirror/rsync
PORTDIR=/usr/portage
BASE_PATH=$(readlink -f "${BASH_SOURCE[0]%/*}")
PORTAGE_BASE_PATH=$BASE_PATH/misc/portage
PYTHONPATH=$PORTAGE_BASE_PATH/pym${PYTHONPATH:+:}${PYTHONPATH}
PATH=$PORTAGE_BASE_PATH/bin${PATH:+:}${PATH}
PORTAGE_CONFIGROOT=$BASE_PATH/misc/config_root
FEATURES="strict -metadata-transfer -fixpackages"
BASE="/home/distfiles"
PORTAGE_DEPCACHEDIR=$BASE/depcache
export FEATURES PORTDIR PORTAGE_DEPCACHEDIR PYTHONPATH PORTAGE_CONFIGROOT

START_HOUR=$(date '+%H')
NEW_REPORT=$(($(date '+%k') < 4))

MIRRORDIST="nice -n 5 ${PORTAGE_BASE_PATH}/bin/emirrordist"
LOG_PATH="${BASE}/logs"
DATA_PATH="${BASE}/mirror-dist-data/"
DIST_PATH="/mnt/distfiles/distfiles"
DIST_LOCAL_PATH=/mnt/distfiles/distfiles-local
DIST_LOCAL_DEATH_DAYS=30
TMP_PATH=/mnt/distfiles/tmp
PURGATORY_DIR="/mnt/distfiles/distfiles-purgatory"
PURGATORY_DB="${DATA_PATH}/purgatory.db"
PURGATORY_DEATH_WATCH_INTERVAL=$((7 * 24 * 60 * 60))
DEATH_WATCH_INTERVAL=$((3 * 24 * 60 * 60))
FETCH_LOGSDIR="${BASE}/last-individual-fetch-logs/"
WHITELIST_DIR="/mnt/distfiles/distfiles-whitelist"
OPTS_MIRRORDIST_WHITELIST=""

OPTS_MIRRORDIST_LOG="--failure-log ${LOG_PATH}/failure.log
	--success-log ${LOG_PATH}/success.log
	--scheduled-deletion-log ${LOG_PATH}/scheduled-deletion.log"

# people do stupid things.  make this check tight.
if [[ -n $FETCH_LOGSDIR ]]; then
	[[ -d $FETCH_LOGSDIR ]] && rm "${FETCH_LOGSDIR}"/* &> /dev/null
fi

# people do stupid things.  make this check tight.
[[ -n $TMP_PATH ]] && rm "${TMP_PATH}"/* &> /dev/null

if [[ -d $WHITELIST_DIR ]]; then
	for x in "${WHITELIST_DIR}"/*; do
		[[ ! -f ${x} ]] && continue
		OPTS_MIRRORDIST_WHITELIST+=" --whitelist-from ${x}"
	done
	for x in "${DATA_PATH}"/*.whitelist; do
		OPTS_MIRRORDIST_WHITELIST+=" --whitelist-from ${x}"
	done
fi

OPTS_MIRRORDIST="--mirror --delete \
	--deletion-db ${DATA_PATH}/deathlist.db \
	--deletion-delay ${DEATH_WATCH_INTERVAL} \
	--distfiles-db ${DATA_PATH}/url.db \
	${PORTAGE_CONFIGROOT:+--config-root ${PORTAGE_CONFIGROOT}} \
	${PORTDIR:+--portdir ${PORTDIR}} \
	${TMP_PATH:+--temp-dir ${TMP_PATH}} \
	${DIST_PATH:+--distfiles ${DIST_PATH}} \
	${DIST_LOCAL_PATH:+--distfiles-local ${DIST_LOCAL_PATH}} \
	${PURGATORY_DIR:+--recycle-dir ${PURGATORY_DIR}} \
	${PURGATORY_DB:+--recycle-db ${PURGATORY_DB}} \
	${PURGATORY_DEATH_WATCH_INTERVAL:+--recycle-deletion-delay ${PURGATORY_DEATH_WATCH_INTERVAL}} \
	${MIRROR_OVERRIDES:+--mirror-overrides ${MIRROR_OVERRIDES}} \
	--jobs 30 \
	--load-average 2.0 \
	${FETCH_LOGSDIR:+--fetch-log-dir ${FETCH_LOGSDIR}} \
	${OPTS_MIRRORDIST_WHITELIST} \
	${DEBUGGING:+--dry-run -vv} \
	--mirror-skip gentoo \
	--restrict-mirror-exemptions gentoo"

echo "---------------------------------------------------" >> "${LOG_PATH}/times.log"
echo "START emirrordist  $(date -u)" >> "${LOG_PATH}/times.log"
rm "${LOG_PATH}/failure.log" &> /dev/null
if [[ $NEW_REPORT == 1 ]] ; then
	# 0-4 utc
	if [[ ! -f ${LOG_PATH}/.report_timestamp || -n $(find "${LOG_PATH}/.report_timestamp" -ctime +0 2>/dev/null) ]] ; then
		rm -f "${LOG_PATH}"/{success.log,general.log,.report_timestamp}
		> "${LOG_PATH}/.report_timestamp"
	fi
fi

[[ ${DEBUGGING} ]] && echo ${MIRRORDIST} ${OPTS_MIRRORDIST_LOG} ${OPTS_MIRRORDIST}
${MIRRORDIST} ${OPTS_MIRRORDIST_LOG} ${OPTS_MIRRORDIST}  >> "${LOG_PATH}/emirrordist.out" 2>> "${LOG_PATH}/emirrordist.err"
ret=$?
echo "END   emirrordist  $(date -u)" >> "${LOG_PATH}/times.log"
if [[ $? != 0 ]]; then
	echo "non zero exit code from emirrordist, bailing!" >> "${LOG_PATH}/times.log"
	echo "---------------------------------------------------" >> "${LOG_PATH}/times.log"
	exit $(($?))
fi

echo "START death-list   $(date -u)" >> "${LOG_PATH}/times.log"
find "$DIST_LOCAL_PATH" -type f -ctime +$DIST_LOCAL_DEATH_DAYS -exec rm {} \;
echo "END   death-list   $(date -u)" >> "${LOG_PATH}/times.log"

echo "START hardlinks    $(date -u)" >> "${LOG_PATH}/times.log"
${BASE}/scripts/hardlinker.py "${DIST_LOCAL_PATH}" "${DIST_PATH}" >> "${LOG_PATH}/hardlinker.out" 2>> "${LOG_PATH}/hardlinker.err"
if [[ -s ${LOG_PATH}/hardlinker.out ]] ; then
	# filter out noise
	sed -e 's|^hardlink: timestamp.dev$||' \
		-e '/^$/d' -i "${LOG_PATH}/hardlinker.out"
fi
echo "END   hardlinks    $(date -u)" >> "${LOG_PATH}/times.log"

${BASE}/scripts/gen-report-xml.py ${LOG_PATH}/failure.log ${LOG_PATH}/success.log ${LOG_PATH}/scheduled-deletion.log '%Y-%m-%d' \
	${DIST_PATH} ${BASE}/reports/failure.xml
${BASE}/scripts/gen-report-whitelist-xml.py "$WHITELIST_DIR" "${BASE}/reports/whitelists.xml"
exit 0
